#include <resolv.h>

enum {
	NSU_PREREQ_YXRRSET,   /* RRset exists (value independent) */
	NSU_PREREQ_YXRRSETV,  /* RRset exists (value dependent) */
	NSU_PREREQ_NXRRSET,   /* RRset does not exist */
	NSU_PREREQ_YXDOMAIN,  /* Name is in use */
	NSU_PREREQ_NXDOMAIN   /* RRset does not exist */
};

enum {
	NSU_UPDATE_ADD,    /* Add to an RRset */
	NSU_UPDATE_DEL,    /* Delete an RRset */
	NSU_UPDATE_DELV,   /* Delete an RR from an RRset */
	NSU_UPDATE_DELALL  /* Delete all RRsets from a name */
};

typedef struct __nsu_rr {
	int op; /* One of the above codes */
	int type; /* RR type */
	char name[NS_MAXDNAME];
	int ttl;
	union __nsu_rdata {
		struct in_addr d_a;     /* ns_t_a */
		char *d_name;           /* ns_t_ns, ns_t_cname, ns_t_ptr */
		char *d_txt;            /* ns_t_txt */
		struct {
			unsigned short pref;
			char *name;
		} d_mx;                 /* ns_t_mx */
#if 0
		struct in6_addr d_aaaa; /* ns_t_aaaa */
		struct {
			uint16_t priority;
			uint16_t weight;
			uint16_t port;
			char *target;
		} d_srv;                /* ts_t_srv */
#endif
		struct {
			uint16_t rdlength;
			u_char *rdata;
		} d_any;                /* Any other type */
	} data;
} nsu_rr;

typedef struct {
	unsigned        id :16;         /*%< query identification number */
#if BYTE_ORDER == BIG_ENDIAN
	/* fields in third byte */
	unsigned        qr: 1;          /*%< response flag */
	unsigned        opcode: 4;      /*%< purpose of message */
	unsigned        z0: 3;          /*%< reserved */
	/* fields in fourth byte */
	unsigned        z1: 4;          /*%< reserved */
	unsigned        rcode :4;       /*%< response code */
#else
	/* fields in third byte */
	unsigned        z0: 3;          /*%< reserved */
	unsigned        opcode: 4;      /*%< purpose of message */
	unsigned        qr: 1;          /*%< response flag */
	/* fields in fourth byte */
	unsigned        rcode :4;       /*%< response code */
	unsigned        z1: 4;          /*%< reserved */
#endif
	/* remaining bytes */
	unsigned        zocount: 16;    /*%< number of RRs in the Zone Section */
	unsigned        prcount: 16;    /*%< number of RRs in the Prerequisite Section */
	unsigned        upcount: 16;    /*%< number of RRs in the Update Section. */
	unsigned        adcount: 16;    /*%< number of RRs in the Additional Data Section */
} UPDATE_HEADER;

struct tsig_alg;

struct nsu_signer {
	char *name;
	struct tsig_alg const *alg;
	unsigned short fudge;
	char *ctx;
	unsigned char *digest;
};

char const *tsig_alg_name(struct tsig_alg const *alg);
size_t tsig_alg_digest_size(struct tsig_alg const *alg);

void nsu_signer_set_key(struct nsu_signer *sgn, u_char const *key, size_t len);
void nsu_signer_update(struct nsu_signer *sgn, u_char const *data, size_t len);
void nsu_signer_digest(struct nsu_signer *sgn, u_char *data, size_t len);

int nsu_str_to_type(char const *str);
char const *nsu_type_to_str(int type);

int nsu_nmkquery(res_state statp,
		 char const *zone,
		 nsu_rr *prereq, int num_prereq,
		 nsu_rr *update, int num_update,
		 struct nsu_signer *tsig,
		 u_char *buf, int buflen);
int nsu_mkquery(char const *zone,
		nsu_rr *prereq, int num_prereq,
		nsu_rr *update, int num_update,
		struct nsu_signer *tsig,
		u_char *buf, int buflen);


